#!/opt/cloudlinux/venv/bin/python3 -bb

import requests
import argparse
import logging
import sys
import os
import yaml
import subprocess

TOKEN_PATH = '/var/cln_mock_adapter/debug_jwt.token'
SYSTEM_ID_PATH = '/var/cln_mock_adapter/debug_system_id'
CLN_URL_PATH = '/var/cln_mock_adapter/cln_url'
END_USER_CONF = '/etc/sysconfig/endservertools.yml'
CLIENT_ID = 1

logging.basicConfig(stream=sys.stdout,
                    level=logging.INFO,
                    format='%(asctime)s %(levelname)s:%(name)s:%(message)s')


def main():
    parser = argparse.ArgumentParser(prog="adapt-enduser-vm", add_help=True,
                                     description="adapt enduser vm")
    subparsers = parser.add_subparsers(dest='subparsers')
    register = subparsers.add_parser('register', help='register server')
    subparsers.add_parser('unregister', help='unregister server from cln mock')
    subparsers.add_parser('refresh_jwt', help='refreshes jwt token')
    register.add_argument('-c', '--client-id',
                          help='specify client_id',
                          dest='client_id',
                          default=CLIENT_ID)
    register.add_argument('-l', '--cln-mock-url',
                          help='cln mock url, format: ip:port',
                          dest='cln_mock_url',
                          required=True)
    register.add_argument('-p', '--pushgateway-address',
                          help='pushgateway address, format: ip:port',
                          dest='pushgateway_address',
                          required=True)
    opts = parser.parse_args()
    if opts.subparsers == 'register':

        # backup file if it was created manually before adapting
        if os.path.exists(END_USER_CONF):
            os.rename(END_USER_CONF, f'{END_USER_CONF}.bak')

        _save(yaml.dump({
            'jwt_token_file': TOKEN_PATH,
            'pushgateway_address': opts.pushgateway_address
        },
            default_flow_style=False),
            END_USER_CONF)
        _save(opts.cln_mock_url, CLN_URL_PATH)
        register_server(opts.cln_mock_url, opts.client_id)
    elif opts.subparsers == 'unregister':
        unregister_server()
    elif opts.subparsers == 'refresh_jwt':
        refresh_jwt()
    else:
        raise NotImplementedError('Unsupported param passed')


def register_server(url, client_id):
    if os.path.exists(SYSTEM_ID_PATH):
        logging.warning('Your server is already registered!')
        return
    try:
        r = requests.post(f'{url}/servers/', data={'client_id': client_id,
                                                          'hostname': f'server{client_id}.com',
                                                          'ip': get_ip_address()})
    except requests.RequestException as e:
        logging.error('Server for client %s was not registered due to %s', client_id, e)
        return
    if not r.ok:
        logging.error('Error response from cln mock server, '
                      'status code: %s, response %s', r.status_code, r.text)
        return

    data = r.json()
    jwt = data.get('jwt')
    system_id = data.get('system_id')
    client_id = data.get('client_id')

    _save(jwt.encode(), TOKEN_PATH, 'wb')
    _save(system_id, SYSTEM_ID_PATH)

    subprocess.call(['/usr/share/cloudlinux/cl_plus/setup_clplus'])
    subprocess.call(['service', 'cl_plus_sender', 'restart'])

    logging.info(f'YOU SERVER IS ADDED TO CLN MOCK AND READY TO WORK!')
    logging.info(f'SYSTEM_ID: {system_id}')
    logging.info(f'CLIENT_ID: {client_id}')
    logging.info(f'JWT TOKEN: {jwt}')
    logging.info(f'TOKEN FILE PATH: {TOKEN_PATH}')


def unregister_server():
    cln_url = _get_from_file(CLN_URL_PATH).strip()
    system_id = _get_from_file(SYSTEM_ID_PATH)
    try:
        r = requests.delete(f'{cln_url}/servers/{system_id}/')
    except requests.RequestException as e:
        logging.error('Server %s was not unregistered due to %s', system_id, e)
        return
    if not r.ok:
        logging.error('Error response from cln mock server, '
                      'status code: %s, response %s', r.status_code, r.text)
        return
    if os.path.exists(SYSTEM_ID_PATH):
        os.remove(SYSTEM_ID_PATH)
    if os.path.exists(TOKEN_PATH):
        os.remove(TOKEN_PATH)

    if os.path.exists(f'{END_USER_CONF}.bak'):
        os.rename(f'{END_USER_CONF}.bak', END_USER_CONF)
    elif os.path.exists(END_USER_CONF):
        os.remove(END_USER_CONF)

    if os.path.exists(CLN_URL_PATH):
        os.remove(CLN_URL_PATH)
    logging.info(f'Server {system_id} was unregistered from cln mock')


def refresh_jwt():
    cln_url = _get_from_file(CLN_URL_PATH).strip()
    system_id = _get_from_file(SYSTEM_ID_PATH)
    data = requests.get(f'{cln_url}/servers/{system_id}/').json()
    jwt = data.get('jwt')
    _save(jwt.encode(), TOKEN_PATH, 'wb')


def _save(what, where, mode='w'):
    logging.info(f'Creating file {where}')
    try:
        with open(where, mode) as f:
            f.write(what)
    except (OSError, IOError) as e:
        logging.error('Error while saving: %s', e)


def _get_from_file(path):
    data = None
    try:
        with open(path) as f:
            data = f.read()
    except (OSError, IOError) as e:
        logging.error('Failed to read %s, error: %s', path, e)
    return data


def get_ip_address(ifname=None):
    """
    Retrieve local IP address
    (same method as in test python_lib in QA)
    :param ifname: Network interface name
    :return: IP address name
    """
    import socket
    import fcntl
    import struct
    if not ifname:
        # No network interface name provided, try to detect it
        with open('/proc/net/dev', 'rb') as net_dev:
            for line in net_dev:
                line = line.strip()
                if line.startswith(b'eth'):
                    ifname = line.split(b':')[0]
    if not ifname:
        ifname = b'eth0'
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    return socket.inet_ntoa(fcntl.ioctl(s.fileno(), 0x8915, struct.pack('256s', ifname[:15]))[20:24])


if __name__ == "__main__":
    main()
